<?php

use Faker\Factory;
use Faker\Generator;
use Helper\WebDriverHelper;
use Step\Acceptance\Emails;
use Step\Acceptance\EmailsTester;
use Step\Acceptance\ListView;

class EmailsCest
{
    /**
     * @var Generator $fakeData
     */
    protected $fakeData;

    /**
     * @var integer $fakeDataSeed
     */
    protected $fakeDataSeed;

    /**
     * @param AcceptanceTester $I
     */
    public function _before(AcceptanceTester $I)
    {
        if (!$this->fakeData) {
            $this->fakeData = Factory::create();
        }

        $this->fakeDataSeed = rand(0, 2048);
        $this->fakeData->seed($this->fakeDataSeed);
    }

    /**
     * @param AcceptanceTester $I
     * @param ListView $listView
     * @param EmailsTester $emails
     * @param WebDriverHelper $webDriverHelper
     *
     * As an administrator I want to view the emails module.
     */
    public function testScenarioViewEmailsModule(
        AcceptanceTester $I,
        ListView $listView,
        EmailsTester $emails,
        WebDriverHelper $webDriverHelper
    ) {
        $I->wantTo('View the emails module for testing');

        $I->amOnUrl(
            $webDriverHelper->getInstanceURL()
        );

        // Navigate to emails list-view
        $I->loginAsAdmin();
        $emails->gotoEmails();
        $listView->waitForListViewVisible();

        $I->see('Emails', '.module-title-text');
    }

    /**
     * @param AcceptanceTester $I
     * @param WebDriverHelper $webDriverHelper
     *
     * As an administrator I want to view an email body and check that it's not cached.
     */
    public function testScenarioViewEmailBodyHTML(
        AcceptanceTester $I,
        WebDriverHelper $webDriverHelper
    ) {

        // TODO: Refactor

        $I->wantTo('View the HTML of two emails');

        $I->amOnUrl(
            $webDriverHelper->getInstanceURL()
        );

        $I->loginAsAdmin();

        // Check for HTML caching issue
        $I->amOnUrl($webDriverHelper->getInstanceURL() . '/index.php?module=Emails&action=DetailView&record=eae65b87-6852-e43c-4213-5b213b39f2aa');
        $I->see('Test Description 1');
        $I->amOnUrl($webDriverHelper->getInstanceURL() . '/index.php?module=Emails&action=DetailView&record=eae65b87-6852-e43c-4213-5b213b39f2ab');
        $I->see('Test Description 2');
    }
}
